@@ -1,9 +1,9 @@
 /* posh: src/builtins.c
-   Copyright (C) 2003, 2004, 2005, 2006  Clint Adams
+   Copyright (C) 2003-2020  Clint Adams
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -79,7 +79,7 @@ int posh_builtin_brkcont(int argc, char **argv, int flags)
 		 */
 		if (n == quit) {
 			warningf(TRUE, "%s: cannot %s", argv[0], argv[0]);
-			return 0; 
+			return 0;
 		}
 		/* POSIX says if n is too big, the last enclosing loop
 		 * shall be used.  Doesn't say to print an error but we
@@ -238,9 +238,59 @@ int posh_builtin_colon(int UNUSED(argc), char UNUSED(**argv), int flags)
 }
 
 /* Deal with command -vV - command -p dealt with in comexec() */
-int posh_builtin_command(int UNUSED(argc), char UNUSED(**argv), int UNUSED(flags))
+int posh_builtin_command(int argc, char **argv, int UNUSED(flags))
 {
-	return 1;
+	int optc;
+	int capital_V = 0;
+	struct tbl *tp;
+
+	while ((optc = getopt(argc, argv, "+vV")) != -1) {
+		switch (optc) {
+			case 'v':
+				break;
+			case 'V':
+				capital_V = 1;
+				break;
+			default:
+				bi_errorf("wat");
+		}
+	}
+
+	argv += optind;
+	argc -= optind;
+
+	if(!argv[0])
+		return 1;
+
+	tp = findcom(argv[0], FC_BI|FC_FUNC|FC_PATH);
+	if(!tp)
+		return 1;
+
+	switch(tp->type) {
+		case CSHELL:
+			if(capital_V)
+				shprintf("shell builtin: %s\n", tp->name);
+			else
+				shprintf("%s\n", tp->name);
+			break;
+		case CFUNC:
+			if(capital_V)
+				shprintf("function: %s\n", tp->name);
+			else
+				shprintf("%s\n", tp->name);
+			break;
+		case CEXEC:
+			if(!(tp->flag & ISSET))
+				return 1;
+
+			if(capital_V)
+				shprintf("executable: %s\n", tp->val.s);
+			else
+				shprintf("%s\n", tp->val.s);
+			break;
+	}
+
+	return 0;
 }
 
 int posh_builtin_dot(int UNUSED(argc), char UNUSED(**argv), int UNUSED(flags))
@@ -406,7 +456,7 @@ int posh_builtin_exec(int UNUSED(argc), char UNUSED(**argv), int UNUSED(flags))
 			if (e->savefd[i] > 0)
 				close(e->savefd[i]);
 		}
-		e->savefd = NULL; 
+		e->savefd = NULL;
 	}
 	return 0;
 }
